# XHTML formatting support
# Adapted from Timeclock program.

class String
  def unindent
    gsub(/^\s*\./m, '')
  end
end

module HtmlFormatting
  Prolog = %Q{.<!DOCTYPE html PUBLIC
                    .           "-//W3C//DTD XHTML 1.0 Transitional//EN"
                    .           "DTD/xhtml1-transitional.dtd"	>
                   }.unindent

  def javascript
    nil
  end

  def page(page_name, *body_guts)
    Prolog +
      html(head(title(page_name),
                javascript),
           body(h1(p({:align => "center"}, page_name)),
                *body_guts))
  end

  # tags
  def self.def_tag(name, type)
    class_eval "def #{name}(*args)
                        #{type}('#{name}', *args)
                      end"
  end

  def_tag :html, :expansive
  def_tag :head, :expansive
  def_tag :body, :expansive
  def_tag :p, :expansive
  def_tag :form, :expansive
  def_tag :input, :tight
  def_tag :table, :expansive
  def_tag :tr, :expansive
  def_tag :td, :expansive
  def_tag :h1, :tight
  def_tag :b, :tight
  def_tag :div, :expansive
  def_tag :title, :tight
  def_tag :ul, :expansive
  def_tag :li, :expansive
  def_tag :ol, :expansive

  def br; "<br/>"; end
  
  # Utilities used in constructing nested tag structures.

  def session_id
    'none for demo'
  end

  def command_form(command, *args)
    form({:method => 'get',
           :action => command,
           :id => command,
           :name => command},
         hidden('session', "#{session_id}"),
         *args)
  end

  def submit(name, button_text=name)
    input({:type => 'submit',
            :name => name,
            :value => button_text})
  end

  def hidden(name, value)
    input({:type => 'hidden',
            :name => name,
            :value => value})
  end

  def textfield(name, value = '')
    input({:type => 'text',
            :name => name,
            :value => value,
            :size => '20',
            :maxlength => '100'})
  end
    
  def labeled_textfield(name, value = '')
    "#{name.capitalize}: #{textfield(name, value)} #{br}"
  end
    


  # Low level utilities.

  def rshift(string)
    "  " + string.split("\n").join("\n  ")
  end

  # Split optional hash of attributes away from trailing texts.
  def attributes_and_contents(args)
    args = args.flatten.compact
    all_attributes = {}
    contents = []
    args.each { | arg |
      if arg.is_a? Hash
        all_attributes.merge!(arg)
      else
        contents << arg
      end
    }

    if all_attributes.empty?
      return [], contents
    else
      pairs = all_attributes.to_a
      # sorting makes tests deterministic. It would be terser
      # to sort the constructed string, but that would confusingly
      # put "arg2=foo" before "arg=foo".
      pairs.sort! { | one, two | one[0].to_s<=>two[0].to_s }
      return pairs.collect { | pair | " #{pair[0]}=\"#{pair[1]}\""},
        contents
    end
  end

  def wrap(tag, attribute_text, content_text)
    "<#{tag}#{attribute_text}>#{content_text}</#{tag}>"
  end

  def tight(tag, *args)
    attrs, contents = attributes_and_contents(args)
    wrap(tag, attrs.join, contents.join)
  end

  def expansive(tag, *args)
    attrs, contents = attributes_and_contents(args)
    wrap(tag, attrs.join, 
         $/ + contents.collect { | text | rshift(text) +$/ }.join)
  end


  
end



